version 12
set more off
adopath + ../external/lib/stata/gslab_misc/ado
adopath + ../external/lib/stata/ado
preliminaries, loadglob(../external/lib/input_parameters/input_param.txt)

mata
    function fgls(y,x,v,w,s) {
      gamma = invsym(x'*invsym(w)*x)*(x'*invsym(w)*y)
      epsil = y-x*gamma
      N = rows(x)
      K = rows(gamma)
      g_vce = (N/(N-K))*invsym(x'*x)*x'*(diag(epsil:*epsil)+v)*x*invsym(x'*x)

      sigm2 = 0
      wt = 0 
      for (i=1;i<=N;i++) {
        sigm2 = sigm2+(1/(s+v[i,i]))*((N/(N-K))*epsil[i,1]^2-v[i,i])
        wt = wt+(1/(s+v[i,i]))
      }

      sigm2 = sigm2/wt

      s_vce = 0
      for (i=1;i<=N;i++) {
        s_vce = s_vce+(1/((s+v[i,i])*wt))*(epsil[i,1]^2-v[i,i]-sigm2)^2
      }
      s_vce = s_vce / (N-K)

      st_matrix("r(gamma)",gamma)
      st_matrix("r(g_vce)",g_vce)
      st_matrix("r(sigm2)",sigm2)
      st_matrix("r(s_vce)",s_vce)
    
    }
end

program estimate

    local model "risk scores"
    local outcome "Lnhcccom"

    cap mkdir ../output/`outcome'/
    use "../../Model (`model')/output/`outcome'/alpha", clear
    gen one = 1
    mata X = st_data(.,"a one")
    use "../../Model (`model')/output/`outcome'/beta", clear
    mkmat *, mat(b)
    mata Y = st_data(.,"b")
    use "../../Model (`model')/output/`outcome'/VCE", clear
    mata V = st_data(.,"*")
    mata W_0 = I(rows(X))
    mata fgls(Y,X,V,W_0,0)
    matrix results_0 = vec(r(gamma))\vec(r(g_vce))\r(sigm2)\r(s_vce)

    forval i = 1/10 {
      local sigm2 = results_`=`i'-1'[7,1]
      mata W_`i' = `sigm2'*I(rows(X))+V
      mata fgls(Y,X,V,W_`i',`sigm2')
      matrix results_`i' = vec(r(gamma))\vec(r(g_vce))\r(sigm2)\r(s_vce)
    }  
    matrix results = results_0, results_1, results_10

    forval i = 1/3 {
      matrix results[3,`i'] = sqrt(results[3,`i'])
      matrix results[4,`i'] = .
      matrix results[5,`i'] = .
      matrix results[6,`i'] = sqrt(results[6,`i'])
      matrix results[7,`i'] = sqrt(results[7,`i'])
      matrix results[8,`i'] = sqrt(results[8,`i'])*0.5*results[7,`i']^-1
    }

    matrix gamma = results_10[1..2,1]
    mata gamma = st_matrix("gamma")
    local sigm2 = results_10[7,1]
    mata B = `sigm2'*I(rows(X))*invsym(`sigm2'*I(rows(X))+V)
    mata P = B*Y+(I(rows(X))-B)*X*gamma

    clear
    svmat results
    outsheet using ../output/`outcome'/HLM_ests.csv, replace
    clear
    use "../../Model (`model')/output/`outcome'/vars", clear 
    split var
    drop vars

    gen i = 1
    reshape long vars, i(i)
    drop i _j
    replace vars = substr(vars,10,.)
    destring vars, replace
    rename vars hrrnum
    gen post = .
    mata: st_store(.,"post",P)
    gen beta = .
    mata: st_store(.,"beta",Y)
    gen alpha = .
    mata: X = X[.,1]
    mata: st_store(.,"alpha",X) 
    set obs `=_N+1'
    replace hrrnum = 1 if hrrnum == .
    foreach var of varlist post beta alpha {
       replace `var' = 0 if `var' == .
       qui summ `var'
       qui replace `var' = `var' - r(mean)
    }
    save ../output/`outcome'/posts, replace

end

* Execute
estimate

